(function () {
    if (window.cordova && !(window.cordova instanceof HTMLElement)) {
        throw new Error("cordova already defined");
    }
    var cordovaTemp = {
        /**
         * Plugin callback mechanism.
         */
        // Randomize the starting callbackId to avoid collisions after refreshing or navigating.
        // This way, it's very unlikely that any new callback would get the same callbackId as an old callback.
        callbackId: Math.floor(Math.random() * 2000000000),
        callbacks: {},
        callbackStatus: {
            NO_RESULT: 0,
            OK: 1,
            CLASS_NOT_FOUND_EXCEPTION: 2,
            ILLEGAL_ACCESS_EXCEPTION: 3,
            INSTANTIATION_EXCEPTION: 4,
            MALFORMED_URL_EXCEPTION: 5,
            IO_EXCEPTION: 6,
            INVALID_ACTION: 7,
            JSON_EXCEPTION: 8,
            ERROR: 9
        },

        /**
         * Called by native code when returning the result from an action.
         */
        callbackFromNative: function (callbackId, isSuccess, status, args, keepCallback) {
            try {
                var callback = this.callbacks[callbackId];
                if (callback) {
                    if (isSuccess && status == this.callbackStatus.OK) {
                        callback.success && callback.success(args);
                    } else if (!isSuccess) {
                        callback.fail && callback.fail(args);
                    }
                    /*
                     else
                     Note, this case is intentionally not caught.
                     this can happen if isSuccess is true, but callbackStatus is NO_RESULT
                     which is used to remove a callback from the list without calling the callbacks
                     typically keepCallback is false in this case
                     */
                    // Clear callback if not expecting any more results
                    if (!keepCallback) {
                        delete this.callbacks[callbackId];
                    }
                }
            } catch (err) {
                var msg = "Error in " + (isSuccess ? "Success" : "Error") + " callbackId: " + callbackId + " : " + err;
                console && console.log && console.log(msg);
                throw err;
            }
        },

        androidExec: function (service, action, callbackId, argsJson) {
            return window.prompt(argsJson, "gap:" + JSON.stringify([service, action, callbackId]));
        },

        typeName: function (val) {
            return Object.prototype.toString.call(val).slice(8, -1);
        },

        exec: function (success, fail, service, action, args) {
            // If args is not provided, default to an empty array
            args = args || [];

            //Process any ArrayBuffers in the args into a string.
            for (var i = 0; i < args.length; i++) {
                if (this.typeName(args[i]) == 'ArrayBuffer') {
                    args[i] = window.btoa(String.fromCharCode.apply(null, new Uint8Array(args[i])));
                }
            }
            var callbackId = service + this.callbackId++;
            var argsJson = JSON.stringify(args);
            if (success || fail) {
                this.callbacks[callbackId] = {success: success, fail: fail};
            }
            if(window._cordovaNative){
                window._cordovaNative.exec(service, action, callbackId, argsJson);
            }else{
                this.androidExec(service, action, callbackId, argsJson);
            }
        }
    };
    window.cordova = cordovaTemp;
}());
